@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2004, 2007-2010  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2004, 2007-2010, 2012, 2013, 2016  Internet Systems Consortium, Inc. ("ISC")
 # Copyright (C) 2000-2003  Internet Software Consortium.
 #
 # Permission to use, copy, modify, and/or distribute this software for any
@@ -20,8 +20,8 @@
 #
 # Set up interface aliases for bind9 system tests.
 #
-# IPv4: 10.53.0.{1..7}				RFC 1918
-# IPv6: fd92:7065:b8e:ffff::{1..7}		ULA
+# IPv4: 10.53.0.{1..8}				RFC 1918
+# IPv6: fd92:7065:b8e:ffff::{1..8}		ULA
 #
 
 config_guess=""
@@ -48,24 +48,24 @@ fi
 # on a NFS mounted disk.
 
 case `uname -a` in
-  *HP-UX*) sys=hpux ;;
-  *) sys=`sh $config_guess` ;;
+	*HP-UX*) sys=hpux ;;
+	*) sys=`sh $config_guess` ;;
 esac
 
 case "$2" in
-[0-9]|[1-9][0-9]|[1-9][0-9][0-9]) base=$2;;
-*) base=""
+	[1-9]|[1-9][0-9]|[1-9][0-9][0-9]) base=$2;;
+	*) base="1"
 esac
 
 case "$3" in
-[0-9]|[1-9][0-9]|[1-9][0-9][0-9]) base6=$2;;
-*) base6=""
+	[1-9]|[1-9][0-9]|[1-9][0-9][0-9]) base6=$3;;
+	*) base6=$base
 esac
 
 case "$1" in
 
     start|up)
-	for ns in 1 2 3 4 5 6 7
+	for ns in 1 2 3 4 5 6 7 8
 	do
 		if test -n "$base"
 		then
@@ -87,7 +87,7 @@ case "$1" in
 			ifconfig lo0:$int 10.53.0.$ns netmask 0xffffffff up
 			;;
 		    *-*-solaris2.[8-9]|*-*-solaris2.1[0-9])
-    			/sbin/ifconfig lo0:$int plumb
+			/sbin/ifconfig lo0:$int plumb
 			/sbin/ifconfig lo0:$int 10.53.0.$ns up
 			if test -n "$int6"
 			then
@@ -99,7 +99,7 @@ case "$1" in
 		    *-*-linux*)
 			ifconfig lo:$int 10.53.0.$ns up netmask 255.255.255.0
 			ifconfig lo inet6 add fd92:7065:b8e:ffff::$ns/64
-		        ;;
+			;;
 		    *-unknown-freebsd*)
 			ifconfig lo0 10.53.0.$ns alias netmask 0xffffffff
 			ifconfig lo0 inet6 fd92:7065:b8e:ffff::$ns alias
@@ -131,7 +131,7 @@ case "$1" in
 		    hpux)
 			ifconfig lo0:$int 10.53.0.$ns netmask 255.255.255.0 up
 			ifconfig lo0:$int inet6 fd92:7065:b8e:ffff::$ns up
-		        ;;
+			;;
 		    *-sco3.2v*)
 			ifconfig lo0 alias 10.53.0.$ns
 			;;
@@ -139,7 +139,11 @@ case "$1" in
 			ifconfig lo0 alias 10.53.0.$ns
 			ifconfig lo0 inet6 fd92:7065:b8e:ffff::$ns alias
 			;;
-	            *)
+		    *-cygwin*)
+			echo "Please run ifconfig.bat as Administrator."
+			exit 1
+			;;
+		    *)
 			echo "Don't know how to set up interface.  Giving up."
 			exit 1
 		esac
@@ -147,13 +151,19 @@ case "$1" in
 	;;
 
     stop|down)
-	for ns in 7 6 5 4 3 2 1
+	for ns in 8 7 6 5 4 3 2 1
 	do
 		if test -n "$base"
 		then
 			int=`expr $ns + $base - 1`
 		else
-			int=$ns	
+			int=$ns 
+		fi
+		if test -n "$base6"
+		then
+			int6=`expr $ns + $base6 - 1`
+		else
+			int6=$ns
 		fi
 		case "$sys" in
 		    *-pc-solaris2.5.1)
@@ -174,7 +184,7 @@ case "$1" in
 		    *-*-linux*)
 			ifconfig lo:$int 10.53.0.$ns down
 			ifconfig lo inet6 del fd92:7065:b8e:ffff::$ns/64
-		        ;;
+			;;
 		    *-unknown-freebsd*)
 			ifconfig lo0 10.53.0.$ns delete
 			ifconfig lo0 inet6 fd92:7065:b8e:ffff::$ns delete
@@ -206,7 +216,7 @@ case "$1" in
 		    hpux)
 			ifconfig lo0:$int 0.0.0.0
 			ifconfig lo0:$int inet6 ::
-		        ;;
+			;;
 		    *-sco3.2v*)
 			ifconfig lo0 -alias 10.53.0.$ns
 			;;
@@ -214,7 +224,11 @@ case "$1" in
 			ifconfig lo0 -alias 10.53.0.$ns
 			ifconfig lo0 inet6 fd92:7065:b8e:ffff::$ns delete
 			;;
-	            *)
+		    *-cygwin*)
+			echo "Please run ifconfig.bat as Administrator."
+			exit 1
+			;;
+		    *)
 			echo "Don't know how to destroy interface.  Giving up."
 			exit 1
 		esac
