@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 #
-# Copyright (C) 2004-2008, 2010-2014  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2004-2008, 2010-2016  Internet Systems Consortium, Inc. ("ISC")
 # Copyright (C) 2001  Internet Software Consortium.
 #
 # Permission to use, copy, modify, and/or distribute this software for any
@@ -68,6 +68,7 @@ my $NAMED = $ENV{'NAMED'};
 my $LWRESD = $ENV{'LWRESD'};
 my $DIG = $ENV{'DIG'};
 my $PERL = $ENV{'PERL'};
+my $PYTHON = $ENV{'PYTHON'};
 
 # Start the server(s)
 
@@ -130,7 +131,7 @@ sub check_ports {
 		if (++$tries > 4) {
 			print "$0: could not bind to server addresses, still running?\n";
 			print "I:server sockets not available\n";
-			print "R:FAIL\n";
+			print "I:failed\n";
 			system("$PERL $topdir/stop.pl $testdir"); # Is this the correct behavior?
 			exit 1;
 		}
@@ -151,7 +152,17 @@ sub start_server {
 
 	if ($server =~ /^ns/) {
 		$cleanup_files = "{*.jnl,*.bk,*.st,named.run}";
-		$command = "$NAMED ";
+		if ($ENV{'USE_VALGRIND'}) {
+			$command = "valgrind -q --gen-suppressions=all --num-callers=48 --fullpath-after= --log-file=named-$server-valgrind-%p.log ";
+			if ($ENV{'USE_VALGRIND'} eq 'helgrind') {
+				$command .= "--tool=helgrind ";
+			} else {
+				$command .= "--tool=memcheck --track-origins=yes --leak-check=full ";
+			}
+			$command .= "$NAMED -m none -M external ";
+		} else {
+			$command = "$NAMED ";
+		}
 		if ($options) {
 			$command .= "$options";
 		} elsif (-e $args_file) {
@@ -196,7 +207,17 @@ sub start_server {
 		$pid_file = "named.pid";
 	} elsif ($server =~ /^lwresd/) {
 		$cleanup_files = "{lwresd.run}";
-		$command = "$LWRESD ";
+		if ($ENV{'USE_VALGRIND'}) {
+			$command = "valgrind -q --gen-suppressions=all --num-callers=48 --fullpath-after= --log-file=lwresd-valgrind-%p.log ";
+			if ($ENV{'USE_VALGRIND'} eq 'helgrind') {
+				$command .= "--tool=helgrind ";
+			} else {
+				$command .= "--tool=memcheck --track-origins=yes --leak-check=full ";
+			}
+			$command .= "$LWRESD -m none -M external ";
+		} else {
+			$command = "$LWRESD ";
+		}
 		if ($options) {
 			$command .= "$options";
 		} else {
@@ -213,7 +234,9 @@ sub start_server {
 		$pid_file = "lwresd.pid";
 	} elsif ($server =~ /^ans/) {
 		$cleanup_files = "{ans.run}";
-                if (-e "$testdir/$server/ans.pl") {
+                if (-e "$testdir/$server/ans.py") {
+                        $command = "$PYTHON ans.py 10.53.0.$' 5300";
+                } elsif (-e "$testdir/$server/ans.pl") {
                         $command = "$PERL ans.pl";
                 } else {
                         $command = "$PERL $topdir/ans.pl 10.53.0.$'";
@@ -231,7 +254,7 @@ sub start_server {
 		$pid_file = "ans.pid";
 	} else {
 		print "I:Unknown server type $server\n";
-		print "R:FAIL\n";
+		print "I:failed\n";
 		system "$PERL $topdir/stop.pl $testdir";
 		exit 1;
 	}
@@ -258,7 +281,7 @@ sub start_server {
 	while (!-s $pid_file) {
 		if (++$tries > 140) {
 			print "I:Couldn't start server $server (pid=$child)\n";
-			print "R:FAIL\n";
+			print "I:failed\n";
 			system "kill -9 $child" if ("$child" ne "");
 			system "$PERL $topdir/stop.pl $testdir";
 			exit 1;
@@ -298,7 +321,7 @@ sub verify_server {
 		if (++$tries >= 30) {
 			print `grep ";" dig.out > /dev/null`;
 			print "I:no response from $server\n";
-			print "R:FAIL\n";
+			print "I:failed\n";
 			system("$PERL $topdir/stop.pl $testdir");
 			exit 1;
 		}
