@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2015  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2011-2016  Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -512,7 +512,7 @@ fix_qname_skip_recurse(dns_rpz_zones_t *rpzs) {
 		 * more significant bits are 0. (For instance,
 		 * 0x0700 => 0x00ff, 0x0007 => 0x0000)
 		 */
-		mask = ~(zbits_req | -zbits_req);
+		mask = ~(zbits_req | ((~zbits_req) + 1));
 
 		/*
 		 * As mentioned in (2) above, the zone corresponding to
@@ -601,13 +601,13 @@ adj_trigger_cnt(dns_rpz_zones_t *rpzs, dns_rpz_num_t rpz_num,
 	}
 
 	if (inc) {
-		if (++*cnt == 1) {
+		if (++*cnt == 1U) {
 			*have |= DNS_RPZ_ZBIT(rpz_num);
 			fix_qname_skip_recurse(rpzs);
 		}
 	} else {
-		REQUIRE(*cnt != 0);
-		if (--*cnt == 0) {
+		REQUIRE(*cnt != 0U);
+		if (--*cnt == 0U) {
 			*have &= ~DNS_RPZ_ZBIT(rpz_num);
 			fix_qname_skip_recurse(rpzs);
 		}
@@ -978,34 +978,43 @@ name2data(dns_rpz_zones_t *rpzs, dns_rpz_num_t rpz_num,
 	(void)dns_name_concatenate(&tmp_name, dns_rootname, trig_name, NULL);
 }
 
-/*
- * Find the first differing bit in a key (IP address) word.
+#ifndef HAVE_BUILTIN_CLZ
+/**
+ * \brief Count Leading Zeros: Find the location of the left-most set
+ * bit.
  */
-static inline int
-ffs_keybit(dns_rpz_cidr_word_t w) {
-	int bit;
+static inline unsigned int
+clz(dns_rpz_cidr_word_t w) {
+	unsigned int bit;
 
 	bit = DNS_RPZ_CIDR_WORD_BITS-1;
+
 	if ((w & 0xffff0000) != 0) {
 		w >>= 16;
 		bit -= 16;
 	}
+
 	if ((w & 0xff00) != 0) {
 		w >>= 8;
 		bit -= 8;
 	}
+
 	if ((w & 0xf0) != 0) {
 		w >>= 4;
 		bit -= 4;
 	}
+
 	if ((w & 0xc) != 0) {
 		w >>= 2;
 		bit -= 2;
 	}
+
 	if ((w & 2) != 0)
 		--bit;
+
 	return (bit);
 }
+#endif
 
 /*
  * Find the first differing bit in two keys (IP addresses).
@@ -1024,12 +1033,14 @@ diff_keys(const dns_rpz_cidr_key_t *key1, dns_rpz_prefix_t prefix1,
 	/*
 	 * find the first differing words
 	 */
-	for (i = 0;
-	     bit < maxbit;
-	     i++, bit += DNS_RPZ_CIDR_WORD_BITS) {
+	for (i = 0; bit < maxbit; i++, bit += DNS_RPZ_CIDR_WORD_BITS) {
 		delta = key1->w[i] ^ key2->w[i];
-		if (delta != 0) {
-			bit += ffs_keybit(delta);
+		if (ISC_UNLIKELY(delta != 0)) {
+#ifdef HAVE_BUILTIN_CLZ
+			bit += __builtin_clz(delta);
+#else
+			bit += clz(delta);
+#endif
 			break;
 		}
 	}
@@ -1173,7 +1184,7 @@ search(dns_rpz_zones_t *rpzs,
 				rpzs->cidr = new_parent;
 			else
 				parent->child[cur_num] = new_parent;
-			child_num = DNS_RPZ_IP_BIT(&cur->ip, tgt_prefix+1);
+			child_num = DNS_RPZ_IP_BIT(&cur->ip, tgt_prefix);
 			new_parent->child[child_num] = cur;
 			cur->parent = new_parent;
 			new_parent->set = *tgt_set;
@@ -1342,12 +1353,9 @@ add_name(dns_rpz_zones_t *rpzs, dns_rpz_num_t rpz_num,
 	isc_result_t result;
 
 	/*
-	 * No need for a summary database of names with only 1 policy zone.
+	 * We need a summary database of names even with 1 policy zone,
+	 * because wildcard triggers are handled differently.
 	 */
-	if (rpzs->p.num_zones <= 1) {
-		adj_trigger_cnt(rpzs, rpz_num, rpz_type, NULL, 0, ISC_TRUE);
-		return (ISC_R_SUCCESS);
-	}
 
 	dns_fixedname_init(&trig_namef);
 	trig_name = dns_fixedname_name(&trig_namef);
@@ -1630,7 +1638,7 @@ fix_triggers(dns_rpz_zones_t *rpzs, dns_rpz_num_t rpz_num) {
 	memset(&rpzs->total_triggers, 0, sizeof(rpzs->total_triggers));
 
 #define SET_TRIG(n, zbit, type)						\
-	if (rpzs->triggers[n].type == 0) {				\
+	if (rpzs->triggers[n].type == 0U) {				\
 		rpzs->have.type &= ~zbit;				\
 	} else {							\
 		rpzs->total_triggers.type += rpzs->triggers[n].type;	\
@@ -2019,12 +2027,9 @@ del_name(dns_rpz_zones_t *rpzs, dns_rpz_num_t rpz_num,
 	isc_result_t result;
 
 	/*
-	 * No need for a summary database of names with only 1 policy zone.
+	 * We need a summary database of names even with 1 policy zone,
+	 * because wildcard triggers are handled differently.
 	 */
-	if (rpzs->p.num_zones <= 1) {
-		adj_trigger_cnt(rpzs, rpz_num, rpz_type, NULL, 0, ISC_FALSE);
-		return;
-	}
 
 	dns_fixedname_init(&trig_namef);
 	trig_name = dns_fixedname_name(&trig_namef);
