@@ -31,6 +31,8 @@
 #include <isc/util.h>
 #include <isc/time.h>
 
+#include <pk11/site.h>
+
 #include <dns/keyvalues.h>
 #include <dns/log.h>
 #include <dns/message.h>
@@ -53,6 +55,7 @@
 #endif
 
 #define is_response(msg) (msg->flags & DNS_MESSAGEFLAG_QR)
+#ifndef PK11_MD5_DISABLE
 #define algname_is_allocated(algname) \
 	((algname) != dns_tsig_hmacmd5_name && \
 	 (algname) != dns_tsig_hmacsha1_name && \
@@ -62,9 +65,20 @@
 	 (algname) != dns_tsig_hmacsha512_name && \
 	 (algname) != dns_tsig_gssapi_name && \
 	 (algname) != dns_tsig_gssapims_name)
+#else
+#define algname_is_allocated(algname) \
+	((algname) != dns_tsig_hmacsha1_name && \
+	 (algname) != dns_tsig_hmacsha224_name && \
+	 (algname) != dns_tsig_hmacsha256_name && \
+	 (algname) != dns_tsig_hmacsha384_name && \
+	 (algname) != dns_tsig_hmacsha512_name && \
+	 (algname) != dns_tsig_gssapi_name && \
+	 (algname) != dns_tsig_gssapims_name)
+#endif
 
 #define BADTIMELEN 6
 
+#ifndef PK11_MD5_DISABLE
 static unsigned char hmacmd5_ndata[] = "\010hmac-md5\007sig-alg\003reg\003int";
 static unsigned char hmacmd5_offsets[] = { 0, 9, 17, 21, 25 };
 
@@ -78,6 +92,7 @@ static dns_name_t hmacmd5 = {
 };
 
 LIBDNS_EXTERNAL_DATA dns_name_t *dns_tsig_hmacmd5_name = &hmacmd5;
+#endif
 
 static unsigned char gsstsig_ndata[] = "\010gss-tsig";
 static unsigned char gsstsig_offsets[] = { 0, 9 };
@@ -316,13 +331,16 @@ dns_tsigkey_createfromkey(dns_name_t *name, dns_name_t *algorithm,
 		goto cleanup_key;
 	(void)dns_name_downcase(&tkey->name, &tkey->name, NULL);
 
+#ifndef PK11_MD5_DISABLE
 	if (dns_name_equal(algorithm, DNS_TSIG_HMACMD5_NAME)) {
 		tkey->algorithm = DNS_TSIG_HMACMD5_NAME;
 		if (dstkey != NULL && dst_key_alg(dstkey) != DST_ALG_HMACMD5) {
 			ret = DNS_R_BADALG;
 			goto cleanup_name;
 		}
-	} else if (dns_name_equal(algorithm, DNS_TSIG_HMACSHA1_NAME)) {
+	} else
+#endif
+	if (dns_name_equal(algorithm, DNS_TSIG_HMACSHA1_NAME)) {
 		tkey->algorithm = DNS_TSIG_HMACSHA1_NAME;
 		if (dstkey != NULL && dst_key_alg(dstkey) != DST_ALG_HMACSHA1) {
 			ret = DNS_R_BADALG;
@@ -540,9 +558,12 @@ destroyring(dns_tsig_keyring_t *ring) {
 
 static unsigned int
 dst_alg_fromname(dns_name_t *algorithm) {
+#ifndef PK11_MD5_DISABLE
 	if (dns_name_equal(algorithm, DNS_TSIG_HMACMD5_NAME)) {
 		return (DST_ALG_HMACMD5);
-	} else if (dns_name_equal(algorithm, DNS_TSIG_HMACSHA1_NAME)) {
+	} else
+#endif
+	if (dns_name_equal(algorithm, DNS_TSIG_HMACSHA1_NAME)) {
 		return (DST_ALG_HMACSHA1);
 	} else if (dns_name_equal(algorithm, DNS_TSIG_HMACSHA224_NAME)) {
 		return (DST_ALG_HMACSHA224);
@@ -725,6 +746,7 @@ dns_tsigkey_create(dns_name_t *name, dns_name_t *algorithm,
 	if (length > 0)
 		REQUIRE(secret != NULL);
 
+#ifndef PK11_MD5_DISABLE
 	if (dns_name_equal(algorithm, DNS_TSIG_HMACMD5_NAME)) {
 		if (secret != NULL) {
 			isc_buffer_t b;
@@ -739,7 +761,9 @@ dns_tsigkey_create(dns_name_t *name, dns_name_t *algorithm,
 				if (result != ISC_R_SUCCESS)
 					return (result);
 		}
-	} else if (dns_name_equal(algorithm, DNS_TSIG_HMACSHA1_NAME)) {
+	} else
+#endif
+	if (dns_name_equal(algorithm, DNS_TSIG_HMACSHA1_NAME)) {
 		if (secret != NULL) {
 			isc_buffer_t b;
 
@@ -957,8 +981,9 @@ dns_tsig_sign(dns_message_t *msg) {
 		 * MAC length > 0 in the reply.
 		 */
 
-		ret = dst_context_create2(key->key, mctx,
-					  DNS_LOGCATEGORY_DNSSEC, &ctx);
+		ret = dst_context_create3(key->key, mctx,
+					  DNS_LOGCATEGORY_DNSSEC,
+					  ISC_TRUE, &ctx);
 		if (ret != ISC_R_SUCCESS)
 			return (ret);
 
@@ -1155,7 +1180,6 @@ dns_tsig_sign(dns_message_t *msg) {
 	datalist->rdclass = dns_rdataclass_any;
 	datalist->type = dns_rdatatype_tsig;
 	ISC_LIST_APPEND(datalist->rdata, rdata, link);
-	dns_rdataset_init(dataset);
 	RUNTIME_CHECK(dns_rdatalist_tordataset(datalist, dataset)
 		      == ISC_R_SUCCESS);
 	msg->tsig = dataset;
@@ -1320,7 +1344,11 @@ dns_tsig_verify(isc_buffer_t *source, dns_message_t *msg,
 	ret = dst_key_sigsize(key, &siglen);
 	if (ret != ISC_R_SUCCESS)
 		return (ret);
-	if (alg == DST_ALG_HMACMD5 || alg == DST_ALG_HMACSHA1 ||
+	if (
+#ifndef PK11_MD5_DISABLE
+	    alg == DST_ALG_HMACMD5 ||
+#endif
+	    alg == DST_ALG_HMACSHA1 ||
 	    alg == DST_ALG_HMACSHA224 || alg == DST_ALG_HMACSHA256 ||
 	    alg == DST_ALG_HMACSHA384 || alg == DST_ALG_HMACSHA512)
 	{
@@ -1343,8 +1371,9 @@ dns_tsig_verify(isc_buffer_t *source, dns_message_t *msg,
 		sig_r.base = tsig.signature;
 		sig_r.length = tsig.siglen;
 
-		ret = dst_context_create2(key, mctx,
-					  DNS_LOGCATEGORY_DNSSEC, &ctx);
+		ret = dst_context_create3(key, mctx,
+					  DNS_LOGCATEGORY_DNSSEC,
+					  ISC_FALSE, &ctx);
 		if (ret != ISC_R_SUCCESS)
 			return (ret);
 
@@ -1654,9 +1683,9 @@ tsig_verify_tcp(isc_buffer_t *source, dns_message_t *msg) {
 	}
 
 	if (msg->tsigctx == NULL) {
-		ret = dst_context_create2(key, mctx,
+		ret = dst_context_create3(key, mctx,
 					  DNS_LOGCATEGORY_DNSSEC,
-					  &msg->tsigctx);
+					  ISC_FALSE, &msg->tsigctx);
 		if (ret != ISC_R_SUCCESS)
 			goto cleanup_querystruct;
 
